"""Fetch all Vantage controllers, and print any state changes."""

import argparse
import asyncio
import contextlib
import logging
from typing import Any, Dict

from aiovantage import Vantage, VantageEvent
from aiovantage.models import Master

# Grab connection info from command line arguments
parser = argparse.ArgumentParser(description="aiovantage example")
parser.add_argument("host", help="hostname of Vantage controller")
parser.add_argument("--username", help="username for Vantage controller")
parser.add_argument("--password", help="password for Vantage controller")
parser.add_argument("--debug", help="enable debug logging", action="store_true")
args = parser.parse_args()


def callback(event: VantageEvent, obj: Master, data: Dict[str, Any]) -> None:
    """Print out any state changes."""
    if event == VantageEvent.OBJECT_ADDED:
        print(f"[Master added] '{obj.name}' ({obj.id})")

    elif event == VantageEvent.OBJECT_UPDATED:
        print(f"[Master updated] '{obj.name}' ({obj.id})")
        for attr in data.get("attrs_changed", []):
            print(f"    {attr} = {getattr(obj, attr)}")


async def main() -> None:
    """Run code example."""
    if args.debug:
        logging.basicConfig(level=logging.DEBUG)

    async with Vantage(args.host, args.username, args.password) as vantage:
        # Subscribe to updates for all Vantage controllers
        vantage.masters.subscribe(callback)

        # Fetch all known Vantage controllers
        await vantage.masters.initialize()

        # Keep running for a while
        await asyncio.sleep(3600)


with contextlib.suppress(KeyboardInterrupt):
    asyncio.run(main())



"""Dump all Vantage object interfaces."""
import argparse
import asyncio
import contextlib
from pathlib import Path
from typing import List
from xml.etree import ElementTree

from aiovantage.config_client import ConfigClient

# Grab connection info from command line arguments
parser = argparse.ArgumentParser()
parser.add_argument("host", help="hostname of Vantage controller")
parser.add_argument("--username", help="username for Vantage controller")
parser.add_argument("--password", help="password for Vantage controller")
args = parser.parse_args()

ElementTree.register_namespace("", "http://www.vantagecontrols.com/Vantage")


async def get_interfaces(client: ConfigClient) -> List[str]:
    """Get a list of all available system object types."""
    response = await client.raw_request(
        "IIntrospection", "<GetInterfaces><call/></GetInterfaces>"
    )

    root = ElementTree.fromstring(response)
    return_el = root.find("./GetInterfaces/return")
    assert return_el is not None

    types = []
    for type_el in return_el:
        name = type_el.findtext("./Name")
        if name:
            types.append(name)

    return types


async def get_interface_info(client: ConfigClient, interface: str) -> str:
    """Get the XML schema for the specified object types."""
    response = await client.raw_request(
        "IIntrospection",
        f"<GetInterfaceInfo><call><Interface>{interface}</Interface></call></GetInterfaceInfo>",
    )

    root = ElementTree.fromstring(
        response,
        parser=ElementTree.XMLParser(
            target=ElementTree.TreeBuilder(insert_pis=True, insert_comments=True)
        ),
    )
    return_el = root.find("./GetInterfaceInfo/return/InterfaceInfo")
    assert return_el is not None

    doc = '<?xml version="1.0" encoding="UTF-8"?>\n'
    for child in return_el:
        doc += ElementTree.tostring(child, encoding="unicode")
    return doc


async def main() -> None:
    """Dump all Vantage object interfaces."""
    async with ConfigClient(args.host, args.username, args.password) as client:
        interfaces = await get_interfaces(client)
        for interface in interfaces:
            try:
                schema = await get_interface_info(client, interface)
                file = Path(f"interfaces/{interface}.vidl")
                file.parent.mkdir(parents=True, exist_ok=True)
                file.write_text(schema, encoding="utf-8")
            except ElementTree.ParseError as exc:
                print(f"Failed to get interface for {interface}: {exc}")


with contextlib.suppress(KeyboardInterrupt):
    asyncio.run(main())



"""Dump all Vantage object types."""
import argparse
import asyncio
import contextlib
from pathlib import Path
from typing import List
from xml.etree import ElementTree

from aiovantage.config_client import ConfigClient

# Grab connection info from command line arguments
parser = argparse.ArgumentParser()
parser.add_argument("host", help="hostname of Vantage controller")
parser.add_argument("--username", help="username for Vantage controller")
parser.add_argument("--password", help="password for Vantage controller")
args = parser.parse_args()

ElementTree.register_namespace("vntg", "http://www.vantagecontrols.com/Vantage")
ElementTree.register_namespace("xs", "http://www.w3.org/2001/XMLSchema")


async def get_types(client: ConfigClient) -> List[str]:
    """Get a list of all available system object types."""
    response = await client.raw_request(
        "IIntrospection", "<GetTypes><call/></GetTypes>"
    )

    root = ElementTree.fromstring(response)
    return_el = root.find("./GetTypes/return")
    assert return_el is not None

    types = []
    for type_el in return_el:
        name = type_el.findtext("./Name")
        if name:
            types.append(name)

    return types


async def get_type_info(client: ConfigClient, object_type: str) -> str:
    """Get the XML schema for the specified object types."""
    response = await client.raw_request(
        "IIntrospection",
        f"<GetTypeInfo><call><Type>{object_type}</Type></call></GetTypeInfo>",
    )

    root = ElementTree.fromstring(
        response,
        parser=ElementTree.XMLParser(
            target=ElementTree.TreeBuilder(insert_pis=True, insert_comments=True)
        ),
    )
    return_el = root.find("./GetTypeInfo/return/TypeInfo")
    assert return_el is not None

    doc = '<?xml version="1.0" encoding="UTF-8"?>\n'
    for child in return_el:
        doc += ElementTree.tostring(child, encoding="unicode")
    return doc


async def main() -> None:
    """Dump all Vantage object types."""
    async with ConfigClient(args.host, args.username, args.password) as client:
        object_types = await get_types(client)
        for object_type in object_types:
            try:
                schema = await get_type_info(client, object_type)
                file = Path(f"types/{object_type}.xsd")
                file.parent.mkdir(parents=True, exist_ok=True)
                file.write_text(schema, encoding="utf-8")
            except ElementTree.ParseError as exc:
                print(f"Failed to get schema for {object_type}: {exc}")


with contextlib.suppress(KeyboardInterrupt):
    asyncio.run(main())
